@@ -1160,7 +1160,10 @@ _verbose = False
 _debug = False
 
 # (progn (forward-line 1) (snip-insert-mode "py.f.setdefaultencoding" t) (insert "\n"))
+file_encoding_is_clean = True
 def setdefaultencoding(encoding=None, quiet=False):
+    if file_encoding_is_clean:
+        return
     if encoding is None:
         encoding='utf-8'
     try:
@@ -1205,11 +1208,11 @@ def main(argv):                                            # ||:fnc:||
     #                    help='sum the integers (default: find the max)')
     # |:opt:| add options
     parser.add_argument(
-        '-q', '--quiet', action='store_const', const=-2, dest='debug',
-        default=0, help='suppress warnings')
+        '-q', '--quiet', action='store_const', const=-2,
+        dest='debug', default=0, help='suppress warnings')
     parser.add_argument(
-        '-v', '--verbose', action='store_const', const=-1, dest='debug',
-        default=0, help='verbose test output')
+        '-v', '--verbose', action='store_const', const=-1,
+        dest='debug', default=0, help='verbose test output')
     parser.add_argument(
         '-d', '--debug', nargs='?', action='store', type=int, metavar='NUM',
         default = 0, const = 1,
@@ -1229,8 +1232,8 @@ def main(argv):                                            # ||:fnc:||
     parser.add_argument(
         '--explode', nargs='?', action=AdHocAction, type=str, metavar='DIR',
         default=False, const='__adhoc__',
-        help=('explode script with adhoc in directory DIR'
-              ' (default: `__adhoc__`)'))
+        help='explode script with adhoc in directory DIR'
+        ' (default: `__adhoc__`)')
     parser.add_argument(
         '--extract', nargs='?', action=AdHocAction, type=str, metavar='DIR',
         default=False, const = '.',
@@ -1238,7 +1241,7 @@ def main(argv):                                            # ||:fnc:||
     parser.add_argument(
         '--template', nargs='?', action=AdHocAction, type=str, metavar='NAME',
         default=False, const = '-',
-        help='extract named template to standard output. default NAME is `-`')
+        help='extract named template to standard output. default NAME is ``-``')
     parser.add_argument(
         '-h', '--help', action='store_true',
         help="display this help message")
@@ -1251,15 +1254,15 @@ def main(argv):                                            # ||:fnc:||
         #type=argparse.FileType('r'), default=sys.stdin,
         help='a series of arguments')
     #_parameters = parser.parse_args()
-    (_parameters, _pass_opts) = parser.parse_known_args()
+    (_parameters, _pass_opts) = parser.parse_known_args(argv[1:])
     # generate argparse help
     if _parameters.ap_help:
         parser.print_help()
-        sys.exit(0)
+        return 0
     # standard help
     if _parameters.help:
         sys.stdout.write(__doc__)
-        sys.exit(0)
+        return 0
 
     _debug = _parameters.debug
     if _debug > 0:
@@ -1274,7 +1277,7 @@ def main(argv):                                            # ||:fnc:||
     _parameters.quiet = _quiet
 
     if _debug:
-        cmd_line = sys.argv
+        cmd_line = argv
         sys.stderr.write(sformat(
                 "{0}{3:^{1}} {4:<{2}s}: ]{5!s}[\n",
                 globals()['dbg_comm'] if 'dbg_comm' in globals() else '# ',
@@ -1290,13 +1293,16 @@ def main(argv):                                            # ||:fnc:||
     adhoc_export = (_parameters.explode or _parameters.extract)
     adhoc_op = (_parameters.implode or adhoc_export or _parameters.template)
     if adhoc_op:
-        adhoc_have_rt = 'RtAdHoc' in globals()
-        adhoc_compile = (
-            _parameters.implode
-            or (adhoc_export and not adhoc_have_rt))
-
+        file_ = __file__
         source = None
-        if adhoc_compile or not adhoc_have_rt:
+
+        have_adhoc = 'AdHoc' in globals()
+        have_rt_adhoc = 'RtAdHoc' in globals()
+
+        # shall adhoc be imported
+        if _parameters.implode or not have_rt_adhoc:
+            # shall this file be compiled
+            adhoc_compile = not (have_rt_adhoc)
             os_path = os.defpath
             for pv in ('PATH', 'path'):
                 try:
@@ -1310,69 +1316,72 @@ def main(argv):                                            # ||:fnc:||
                     continue
                 if path_dir not in sys.path:
                     sys.path.append(path_dir)
-            import adhoc
-            AdHoc = adhoc.AdHoc
+            if not have_adhoc:
+                import adhoc
+                AdHoc = adhoc.AdHoc
         else:
+            adhoc_compile = False
             AdHoc = RtAdHoc
 
         AdHoc.quiet = _quiet
         AdHoc.verbose = _verbose
         AdHoc.debug = _debug
-        AdHoc.include_path.append(os.path.dirname(__file__))
-        ah = AdHoc()
+        AdHoc.include_path.append(os.path.dirname(file_))
+
         if adhoc_compile:
-            source = ah.compileFile(__file__)
+            ah = AdHoc()
+            source = ah.compileFile(file_)
         else:
-            source = AdHoc.read_source(__file__)
+            source = AdHoc.read_source(file_)
 
         # implode
         if _parameters.implode:
             # @:adhoc_enable:@
             # if not _quiet:
             #     map(sys.stderr.write,
-            #         ["warning: ", os.path.basename(__file__),
+            #         ["warning: ", os.path.basename(file_),
             #          " already imploded!\n"])
             # @:adhoc_enable:@
             sys.stdout.write(source)
-            exit(0)
-
         # explode
-        if _parameters.explode:
+        elif _parameters.explode:
             AdHoc.export_dir = _parameters.adhoc_arg
-            AdHoc.export(__file__, source)
-            exit(0)
-
+            AdHoc.export(file_, source)
         # extract
-        if _parameters.extract:
+        elif _parameters.extract:
             AdHoc.extract_dir = _parameters.adhoc_arg
-            AdHoc.extract(__file__, source)
-            exit(0)
-
+            AdHoc.extract(file_, source)
         # template
-        if _parameters.template:
+        elif _parameters.template:
             template_name = _parameters.adhoc_arg
             if not template_name:
                 template_name = '-'
             if template_name == 'list':
-                sys.stdout.write('\n'.join(AdHoc.template_table(__file__, source)))
+                sys.stdout.write(
+                    '\n'.join(AdHoc.template_table(file_, source)) + '\n')
             else:
-                template = AdHoc.get_named_template(template_name, __file__, source)
-                sys.stdout.write(re.sub('\n\n+', '\n', template))
-            exit(0)
+                template = AdHoc.get_named_template(
+                    template_name, file_, source)
+                sys.stdout.write(template)
+
+        # restore for subsequent calls to main
+        if not have_adhoc:
+            del(AdHoc)
+        return 0
 
     # run doc tests
     if _parameters.test:
         import doctest
         doctest.testmod(verbose = _verbose)
-        exit(0)
+        return 0
 
     # |:opt:| handle options
     run(_parameters, _pass_opts)
 
 if __name__ == "__main__":
     #sys.argv.insert(1, '--debug') # |:debug:|
-    main(sys.argv)
-    sys.exit()
+    result = main(sys.argv)
+    sys.exit(result)
 
     # |:here:|
 
